#include <config.h>
#include <stdlib.h>
#include "tools.h"
#include "data.h"


/*
 * the variables
 */
static struct hash *vars;


/*
 * initializes the variable hash
 */
void
var_init ()
{
  vars = hash_new ();
}


/*
 * removes the variable hash
 */
void
var_done ()
{
  struct hash_it *i = NULL;

  while (hash_iterate (&i, vars))
    {
      struct sl *l = (struct sl *) i->value;
      sl_clear (l);
      free_save (l);
    }

  hash_clear (vars);
  free_save (vars);
}


/*
 * iterates the var list
 */
int
var_iterate (struct var_it **it)
{
  /* first element */
  if (*it == NULL)
    {
      *it = (struct var_it *) malloc (sizeof(struct var_it));
      (*it)->it = NULL;
    }

  /* last element */
  if (!hash_iterate (&(*it)->it, vars))
    {
      free_save (*it);
      *it = NULL;
      return 0;
    }

  /* not the last element */
  else
    {
      (*it)->name = (*it)->it->key;
      (*it)->var = (struct sl *) (*it)->it->value;
      return 1;
    }
}


/*
 * stops an iteration
 */
void
var_it_stop (struct var_it **it)
{
  /* free iterator if necessary */
  if (*it != NULL)
    {
      /* free iterator */
      hash_it_stop (&(*it)->it);
      free_save (*it);
      *it = NULL;
    }
}


/*
 * returns the string list of a variable
 */
struct sl *
var_get (const char *var)
{
  /* get the list */
  struct sl *t = (struct sl *) hash_get (vars, var);

  /* create it, if necessary */
  if (t == NULL)
    {
      t = sl_new ();
      hash_put (vars, var, t);
    }

  return t;
}


/*
 * the scripts
 */
static struct hash *scripts;


/*
 * initializes the script hash
 */
void
script_init ()
{
  scripts = hash_new ();
}


/*
 * removes the script hash
 */
void
script_done ()
{
  struct hash_it *i = NULL;

  while (hash_iterate (&i, scripts))
    free_save (i->value);

  hash_clear (scripts);
  free_save (scripts);
}


/*
 * iterates the script list
 */
int
script_iterate (struct script_it **it)
{
  /* first element */
  if (*it == NULL)
    {
      *it = (struct script_it *) malloc (sizeof(struct script_it));
      (*it)->it = NULL;
    }

  /* last element */
  if (!hash_iterate (&(*it)->it, scripts))
    {
      free_save (*it);
      *it = NULL;
      return 0;
    }

  /* not the last element */
  else
    {
      (*it)->name = (*it)->it->key;
      (*it)->script = (char *) (*it)->it->value;
      return 1;
    }
}


/*
 * stops an iteration
 */
void
script_it_stop (struct script_it **it)
{
  /* free iterator if necessary */
  if (*it != NULL)
    {
      /* free iterator */
      hash_it_stop (&(*it)->it);
      free_save (*it);
      *it = NULL;
    }
}


/*
 * returns the script of the specified name
 */
char *
script_get (const char *name)
{
  return (char *) hash_get (scripts, name);
}


/*
 * loads the script of the specified name
 */
void
script_put (const char *name, const char *script)
{
  hash_put (scripts, name,
	    str_cpy ((char *) hash_get (scripts, name),
		     script));
}


/*
 * initializes all data
 */
void
data_init ()
{
  var_init ();
  script_init ();
}


/*
 * removes all data
 */
void
data_done ()
{
  var_done ();
  script_done ();
}
