#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "tools.h"
#include "io.h"


/*************************************
 *                                   *
 *   L I B C U R L   V A R I A N T   *
 *                                   *
 ************************************/

#ifdef HAVE_LIBCURL
#include <curl/curl.h>


/*
 * appends ptr to the string given via data
 * (used by the curl variant of readurl)
 */
size_t
readurl_writefunc (void *ptr, size_t size, size_t nmemb, void *data)
{
  *(char **)data = str_catbuf (*(char **)data, ptr, size * nmemb);
  return size * nmemb;
}


/*
 * reads a URL (curl variant)
 */
char *
readurl (const char *url)
{
  char *c = NULL;
  CURL *curl_handle;

  curl_handle = curl_easy_init ();
  curl_easy_setopt (curl_handle, CURLOPT_URL, url);
  curl_easy_setopt (curl_handle, CURLOPT_WRITEFUNCTION, readurl_writefunc);
  curl_easy_setopt (curl_handle, CURLOPT_FILE, (void *)&c);
  curl_easy_perform (curl_handle);
  curl_easy_cleanup (curl_handle);

  return c;
}


/*********************************************
 *                                           *
 *   N O N - L I B C U R L   V A R I A N T   *
 *                                           *
 ********************************************/

#else


/*
 * reads a URL (non-curl variant)
 */
char *
readurl (const char *url)
{
  return NULL;
}


#endif
