#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include "tools.h"
#include "data.h"
#include "loadsave.h"
#include "io.h"


/*
 * displays general information
 */
void
display_info ()
{
  printf ("\n"
	  "   Davin - The DAVIS Interpreter\n"
	  "   =============================\n"
	  "\n"
	  "   Version " VERSION "\n"
	  "\n"
	  "   (C) " COPYRIGHT "\n"
	  "\n"
	  "   This program is free software; you can redistribute it and/or modify\n"
	  "   it under the terms of the GNU General Public License as published by\n"
	  "   the Free Software Foundation; either version 2 of the License, or\n"
	  "   (at your option) any later version.\n"
	  "\n"
	  "   This program is distributed in the hope that it will be useful,\n"
	  "   but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
	  "   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
	  "   GNU General Public License for more details.\n"
	  "\n"
	  "   You should have received a copy of the GNU General Public License\n"
	  "   along with this program; if not, write to the Free Software\n"
	  "   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n"
	  "\n");
}


/*
 * displays usage information
 */
void
display_usage (char *program)
{
  printf ("Usage: %s [options] file...\n"
	  "Interpret the given DAVIS files.\n"
	  "\n"
	  "  -l, --logfile=FILE  output internal logs to FILE (default: davin.log)\n"
	  "  -q, --quiet-log     don't log anything\n"
	  "  -e, --error-log     log only the errors\n"
	  "  -n, --normal-log    log only the important stuff (default)\n"
	  "  -d, --detailed-log  log everything (very big logfile!)\n"
	  "\n"
	  "  -h, --help          display this help\n"
	  "  -v, --version       display general information about this program\n",
	  program);
}


/*
 * the options
 */
static struct option LONG_OPTIONS[] =
{
  {"help",         0, 0, 'h'},
  {"version",      0, 0, 'v'},
  {"logfile",      1, 0, 'l'},
  {"quiet-log",    0, 0, 'q'},
  {"error-log",    0, 0, 'e'},
  {"normal-log",   0, 0, 'n'},
  {"detailed-log", 0, 0, 'd'},
  {0,              0, 0, 0}
};
static char *SHORT_OPTIONS = "hvl:qend";


/*
 * main function
 */
int
main(int argc, char *argv[])
{
  /* default settings */
  log_file = str_cpy (log_file, "davin.log");
  log_mode = L_NORMAL;

  for (;;)
    {
      /* check options */
      switch (getopt_long (argc, argv, SHORT_OPTIONS, LONG_OPTIONS, NULL))
	{
	case 'h':
	  display_usage (argv[0]);
	  return 0;

	case 'v':
	  display_info ();
	  return 0;

	case 'l':
	  log_file = str_cpy (log_file, optarg);
	  break;

	case 'q':
	  log_mode = L_QUIET;
	  break;

	case 'e':
	  log_mode = L_ERROR;
	  break;

	case 'n':
	  log_mode = L_NORMAL;
	  break;

	case 'd':
	  log_mode = L_DETAILED;
	  break;

	case -1:

	  /* check for file parameters */
	  if (argc == optind)
	    {
	      display_usage (argv[0]);
	      return 1;
	    }

	  /* initialize */
	  data_init();
	  set_main_file (argv[optind++]);
	  print2ilog (L_NORMAL, "--- INIT ---");

	  /* load main file */
	  reload ();

	  /* load files */
	  for (; optind < argc; optind++)
	    {
	      load (argv[optind]);
	    }

	  /* run */
	  run ();

	  /* done */
	  print2ilog (L_NORMAL, "--- DONE ---");
	  data_done();
	  load_done();
	  free_save (log_file);

	  /* exit */
	  return 0;
	}
    }
}
